// setwmc: creates a window with desired name and class, and executes a command.
// designed to be used in combination with plank.
//
// usage:
//    setwmc name class command ...
//    example:
//    setwmc Check-Website check-website yad --info --text="setwmc test"
//
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <wait.h>
#include <signal.h>

#define KILL dokill

#define dontkill(p, sig) fprintf(stderr,"line: %d Killing: %d %d\n",__LINE__,p,sig);
#define dokill(p,sig) if (p) { kill(p,sig);}

static  pid_t ff[3]={0,0,0};
static char *class, *name, *me;
static int width=1, height=1;

static void exitall()
{
   KILL(ff[1],SIGKILL);
   KILL(ff[2],SIGKILL);
   raise(SIGKILL);
}
static void sighand(int x)
{
   fprintf(stderr,"%s: Caught signal %d %d .\n",me,x,ff[1]); fflush(0);
   if (ff[1])   // this is parent killing childs
      exitall();
   else
   {
      KILL(ff[0],SIGKILL); // kill parent
      KILL(ff[2],SIGKILL);
      raise(SIGKILL);
   }
}

static int errorhandler(Display *dpy, XErrorEvent *err)
{
   fprintf(stderr,"%s: Exit by errorhandler.\n",me);
   exitall();
   return 0;
}

int main(int argc, char *argv[])
{
   XSetWindowAttributes myat;
   int screen_num;

   if (argc < 4 )
   {
      fprintf(stderr,"usage: %s name class command ...\n",argv[0]);
      exit(1);
   }
   signal(SIGINT,  sighand);
   signal(SIGTERM, sighand);
   signal(SIGHUP,  sighand);

   me    = argv[0];
   name  = argv[1];
   class = argv[2];
   ff[0] = getpid();

   Display *mydisplay;
   mydisplay = XOpenDisplay ("");

   screen_num = DefaultScreen(mydisplay);

   unsigned long valuemask;
   valuemask = 0;
   Window mywindow;
   mywindow = XCreateWindow (mydisplay, RootWindow(mydisplay, screen_num),
	 0, 0, width, height, 0,
	 DefaultDepth(mydisplay, screen_num), InputOutput ,
	 DefaultVisual(mydisplay, screen_num),
	 valuemask, &myat);

   XClassHint class_hints;
   class_hints.res_name  = name;
   class_hints.res_class = class;
   XSetClassHint(mydisplay,mywindow,&class_hints);

   XSizeHints wmsize;
   wmsize.flags = USPosition | USSize;
   XSetWMNormalHints ( mydisplay , mywindow , &wmsize ) ;
   if (1)
   {
      long hints[5] = {2 , 0, 0, 0, 0};
      Atom motif_hints = XInternAtom(mydisplay, "_MOTIF_WM_HINTS", False);
      XChangeProperty(mydisplay, mywindow, motif_hints, motif_hints, 32, PropModeReplace, (unsigned char *)&hints, 5);

   }

   char*window_name = "xsetwmc";
   XTextProperty windowName;
   XStringListToTextProperty(&window_name,1,&windowName);
   XSetWMName(mydisplay,mywindow,&windowName);
   XMapWindow( mydisplay, mywindow);
   XSetErrorHandler(errorhandler);
   XFlush(mydisplay);

   ff[1] = fork();
   if (ff[1]==0)
   {
      // in this process we execute the command
      if(0)
      {
	 int i=0;
	 while (1)
	 {
	    if (argv[i] == 0)
	       break;
	    fprintf(stderr,"%d: %s\n",i,argv[i]);
	    i++;
	 }
      }

      execvp(argv[3],&argv[3]);
   }

   ff[2] = fork();
   if (ff[2] == 0)
   {
      // in this process we check if our window still exists.
      // if not: kill everything.
      Display *dis = XOpenDisplay ("");
      while(1)
      {
	 usleep(100000);
	 XWindowAttributes attr;
	 int rc = XGetWindowAttributes(dis, mywindow, &attr);
	 if(0)
	    fprintf(stderr,"rc:%d %d %d %d %d\n",rc,attr.x,attr.y,attr.width,attr.height); 
	 if (rc != 1)
	 {
	    fprintf(stderr,"%s: Exit by window not available.\n",me);
	    exitall();
	 }
	 if (attr.width != width || attr.height != height)
	 {
	    fprintf(stderr,"%s: Exit by window changed.\n",me);
	    exitall();
	 }
      }
   }

   int wstatus;
   waitpid(ff[1],&wstatus,0);
   KILL(ff[1],SIGKILL);
   KILL(ff[2],SIGKILL);
   XUnmapWindow(mydisplay, mywindow);
   XDestroyWindow(mydisplay, mywindow);
   XCloseDisplay(mydisplay);

   if ( WIFEXITED(wstatus) )
      return WEXITSTATUS(wstatus);
   return 0;
}
